//
//  CCEffectDFInnerGlow.h
//  cocos2d-ios
//
//  Created by Oleg Osin on 9/11/14.
//
//

#import "CCEffect.h"

/**
 * CCEffectDFInnerGlow create an inner glow on any sprite (combined with a distance field).
 *
 */
#if CC_EFFECTS_EXPERIMENTAL

@interface CCEffectDFInnerGlow : CCEffect

/// -----------------------------------------------------------------------
/// @name Accessing Effect Attributes
/// -----------------------------------------------------------------------

/** Color of the sprite (pass a fillColor with with alpha value of 0.0 to see the actual sprite) */
@property (nonatomic, strong) CCColor* fillColor;

/** Color of glow */
@property (nonatomic, strong) CCColor* glowColor;

/** Outline width pixel width of the glow */
@property (nonatomic) int glowWidth;

// Distance Field, generated by SDFGen (https://github.com/osinoleg/SDFGen)
@property (nonatomic, strong) CCTexture* distanceField;

/// -----------------------------------------------------------------------
/// @name Initializing a CCEffectDFInnerGlow object
/// -----------------------------------------------------------------------

/**
 *  Initializes a CCEffectDFInnerGlow.
 *
 *  @return The CCEffectDFInnerGlow object.
 */
-(id)init;

/**
 *  Initializes a CCEffectDFInnerGlow object with the supplied parameters.
 *
 *  @param glowColor Color of the glow, a [CCColor blackColor] will result in an opaque black glow.
 *  @param fillColor Color of the fillColor, a [CCColor blackColor] will result in an opaque black fillColor.
 *  @param glowWidth pixel width of the glow.
 *  @param fieldScale, defined by the distance field generation proccess, for example a distance field that
 *  was generated with a 4096/4096 input and output a 128/128 texture would have a fieldScale of 32 (
 *  input size / output size). Note: this parameter could be automatically calculated if we assume that all
 *  distance fields are generated from a 4096 input.
 *
 *  @return The CCEffectDFInnerGlow object.
 */
-(id)initWithGlowColor:(CCColor*)glowColor fillColor:(CCColor*)fillColor glowWidth:(int)glowWidth fieldScale:(float)fieldScale distanceField:(CCTexture*)distanceField;


/// -----------------------------------------------------------------------
/// @name Creating a CCEffectDFInnerGlow object
/// -----------------------------------------------------------------------

/**
 *  Initializes a CCEffectDFInnerGlow object with the supplied parameters.
 *
 *  @param glowColor Color of the glow, a [CCColor blackColor] will result in an opaque black glow.
 *  @param fillColor Color of the fillColor, a [CCColor blackColor] will result in an opaque black fillColor.
 *  @param glowWidth pixel width of the glow.
 *  @param fieldScale, defined by the distance field generation proccess, for example a distance field that
 *  was generated with a 4096/4096 input and output a 128/128 texture would have a fieldScale of 32 (
 *  input size / output size). Note: this parameter could be automatically calculated if we assume that all
 *  distance fields are generated from a 4096 input.
 *
 *  @return The CCEffectDFInnerGlow object.
 */
+(id)effectWithGlowColor:(CCColor*)glowColor fillColor:(CCColor*)fillColor glowWidth:(int)glowWidth fieldScale:(float)fieldScale distanceField:(CCTexture*)distanceField;

@end

#endif

